/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.Dialog");

dojo.require("jazz.ui.StyledBox");
dojo.require("jazz.ui.styledBox.TrimStyles");
dojo.require("dojo.fx");

(function() {

//
// Aliases
//
var TrimStyles = jazz.ui.styledBox.TrimStyles;

// Constants
var BLOCKER_OPACITY = 0.1;
/*
 * PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * future with minimal or no warning.
 * 
 * @since 0.7
 */
dojo.declare("jazz.ui.Dialog",
	jazz.ui.StyledBox, {
	
	contentNode: null,  //DOM node for contents of dialog
	width: "40%",		//Optional - zero means natural width, but content scroll bar bugs present when height is fixed
	height: "",			//Optional - zero means natural height
	primaryTitle: "",
	secondaryTitle: "",
	trim: TrimStyles.BLUE_ALPHA_SHADOW,
	closable: true,
	headerHoverState: false,
	headerDefaultToHoverState: true,
	showBackground: false,
	modal: true,
	moveable: true,	
	center: true,  		//true: absolutely position this block in the middle of the screen, false: render it where it is inserted in the dom
	closeOnEsc: true,
	iconPath: "",
	destroyContents: false, //causes the dialog to destroy the widget passed in 'contentNode' on close (if it's a widget or the domNode for a widget)
	
	//callback functions
	onClose: null,		//in your handler, return boolean false to veto dialog close.
	onOpen: null,

	postCreate: function() {
		this._lastFocus = dijit.getFocus(this);
		
		// Hitch the user's onOpen callback to the input of _onOpen,
		// then nullify onOpen so that this.inherited doesn't call it 
		// before dialog is appended to the DOM.
		this._onOpen = dojo.hitch(this,this._onOpen,this.onOpen);
		this.onOpen = null;
		
		this.inherited(arguments);
		
		if (this.modal) {
			this.blockerDiv = document.createElement("div");
			this.blockerDiv.className = "jazz-ui-Dialog-blockerDiv";

			var de = document.documentElement;
			var height = de.clientHeight;
			var scrollHeight = de.scrollHeight;
			
			this.blockerDiv.style.height = ((scrollHeight > height) ? scrollHeight : height) + "px";
			this.blockerDiv.style.top = "0";
			this.blockerDiv.style.left = "0";
			this.blockerDiv.style.opacity = "0";
			
			document.body.appendChild(this.blockerDiv);
			
			var anim = dojo.animateProperty({node: this.blockerDiv, duration: 280, 
				properties: 
					{opacity: {start: 0, end: BLOCKER_OPACITY}
				}});
			anim.play();
			
			var access = document.createElement("span");
			access.tabIndex = 0;
			this.connect(access,"onfocus",function() {
				this._headerText.focus();
			});
			this.domNode.appendChild(access);
			access = null;
		}

		if (this.center){
			this._registerUpdateEvents();
			dojo.addClass(this.domNode, "jazz-ui-Dialog-absolute");
			this.domNode.style.height = this.height;
			document.body.appendChild(this.domNode);
			setTimeout(dojo.hitch(this,this.updatePosition),0);
		}

		if (dojo.isIE && this.blockerDiv) this.blockerDiv.style.display = "block";
		
		if (this.moveable) {
			this.events = [];
			this.events = new Array();
			this.events.push(dojo.connect(this._header, "ondragstart", this, this.dragStopEvent));
			this.events.push(dojo.connect(this._header, "onselectstart", this, this.dragStopEvent));
			this.events.push(dojo.connect(document.body,"ondragstart", this, this.dragStopEvent));
			this.events.push(dojo.connect(document.body,"onselectstart", this, this.dragStopEvent));
			this.events.push(dojo.connect(this._headerButtons, "onmousedown", this, this._cancelBubble))
			this._clickEvent = dojo.connect(this._header, "onmousedown", this, "_onMouseDown");
			this._isDragging = false;
			this._header.style.cursor = "move";
		}
		if (this.closeOnEsc) {
			this.events.push(dojo.connect(document.body, "onkeypress", this, this._checkForEsc));
		}
		
		//Stop bleed-through in high contrast mode and IE6 form elements.
		this._backgroundIframe = new dijit.BackgroundIframe(this.domNode);
		dijit.setWaiRole(this._headerText,"dialog");
		this._onOpen(/* onOpen - hitched to input (see above) */);
	},
	_onOpen: function(onOpen) {
		this._headerText.tabIndex = 0;	//make dialog focusable
		dijit.focus(this._headerText);	//Set focus on the dialog
		if (onOpen && typeof onOpen == "function") {
			onOpen();
		}
	},
	_checkForEsc: function (event) {
		if (event.keyCode == 27) {
			this.close();
		}
	},
	_cancelBubble: function(event) {
		event.cancelBubble = true;
	},
	dragStopEvent: function(event) {
		if (event && (this._isDragging || this._initialMouseCoords)) {
			dojo.stopEvent(event);
		}
	},
	_onMouseDown: function(event) {
		dojo.disconnect(this._clickEvent);
		this._unregisterUpdateEvents();
		this._clickEvent = dojo.connect(document.body, "onmouseup", this, this._onMouseUp);
		
		var coords = dojo.coords(this.domNode, true);
		this._initialMouseCoords = {
			x: event.pageX,
			y: event.pageY,
			offsetX: event.pageX - coords.x,
			offsetY: event.pageY - coords.y
		}
		dojo.addClass(document.body,"jazz-ui-moveCursor");
		this._moveEvent = dojo.connect(dojo.doc, "onmousemove", this, this._onMouseMoveInitial);
		this.dragStopEvent(event);
	},
	_onMouseMoveInitial: function(e) {
		var difX = Math.abs(e.pageX - this._initialMouseCoords.x);
		var difY = Math.abs(e.pageY - this._initialMouseCoords.y);
		if (!this._isDragging && (difX >= 3 || difY >= 3)) {
			this._isDragging = true;
			dojo.disconnect(this._moveEvent);
			this._moveEvent = dojo.connect(dojo.doc, "onmousemove", this, this._onMouseMove);
			this._onMouseMove(e);
		} else {
			this.dragStopEvent(e);
		}
	},
	_onMouseMove: function(e) {
		this.dragStopEvent(e);
		var x = e.pageX - this._initialMouseCoords.offsetX;
		var y = e.pageY - this._initialMouseCoords.offsetY;
		this._customPosition = {l: x, t: y};
		this.domNode.style.top = y + "px";
		this.domNode.style.left = x + "px";
	},
	_onMouseUp: function(event) {
		this._isDragging = false;
		dojo.removeClass(document.body,"jazz-ui-moveCursor");
		dojo.disconnect(this._clickEvent);
		dojo.disconnect(this._moveEvent);
		this._clickEvent = dojo.connect(this._header, "onmousedown", this, this._onMouseDown);
		this._registerUpdateEvents();
		
		var de = document.documentElement;
		var scrollTop = de.scrollTop ? de.scrollTop : document.body.scrollTop;
		var scrollLeft = de.scrollLeft ? de.scrollLeft : document.body.scrollLeft;
		if (this._customPosition) {
			this._position = {l: this._customPosition.l - scrollLeft, t: this._customPosition.t - scrollTop};
		}
		this._initialMouseCoords = null;
	},
	_registerUpdateEvents: function(){
		this._onResizeEvent = dojo.connect(window, "onresize", this, this.updatePosition);
		this._onScrollEvent = dojo.connect(window, "onscroll", this, this.updatePosition);		
	},
	_unregisterUpdateEvents: function() {
		dojo.disconnect(this._onResizeEvent);
		dojo.disconnect(this._onScrollEvent);	
	},
	updatePosition: function(event) {
		var de = window.document.documentElement;
		var height = de.clientHeight;
		var width = de.clientWidth;
		var scrollTop = de.scrollTop ? de.scrollTop : window.document.body.scrollTop;
		var scrollLeft = de.scrollLeft ? de.scrollLeft : window.document.body.scrollLeft;
		var scrollHeight = de.scrollHeight ? de.scrollHeight : window.document.body.scrollHeight;
	
		if (!event || event.type != "scroll") {
			this._bodyContainer.style.maxHeight = height - 40 + "px";
			this.domNode.style.maxWidth = width - 40 + "px";
		}
	
		if (this.center) {
			var coords = dojo.coords(this.domNode);
			
			var top, left;
			if (this.moveable && event && this._customPosition) {
				if (event.type == "scroll") {
					top = this._position.t;
					left = this._position.l;
				} else if (event.type == "resize") {
					if (this._position.l + coords.w > width) {
						left = width - coords.w;
					} else {
						left = this._position.l;
						var noAnimX = true;
					}
					
					if (this._position.t + coords.h > height) {
						top = height - coords.h;
					} else {
						top = this._position.t;
						var noAnimY = true;
					}
				}
			} else {
				top = height / 2 - coords.h / 2;
				if (top < 10) 
					top = 10;
				
				left = width / 2 - coords.w / 2;
				if (left < 10) 
					left = 10;
			}
		
			
			if (this.anim && this.anim.status() == "playing") {
				this.anim.stop();
				this._position = {l: coords.l, t: coords.t};
			}
			if (this._position && !(noAnimX && noAnimY)) {
				if (coords.t + coords.h < scrollTop) {
					coords.t = scrollTop - coords.h;
				} else if (coords.t > scrollTop + height) {
					coords.t = scrollTop + height;
				}
				this.anim = dojo.animateProperty({node: this.domNode, duration: 160, 
					properties: 
						{top: {start: coords.t, end: top + scrollTop, unit: "px"},
						left: {start: coords.l, end: left + scrollLeft, unit: "px"}
					}});	
				this.anim.play(); 
			} else {
				dojo.marginBox(this.domNode, {l: left + scrollLeft, t: top + scrollTop});
			}
			this._position = {l: left, t: top};	
		}
		if (this.blockerDiv) {
			this.blockerDiv.style.height = ((scrollHeight > height) ? scrollHeight : height) + "px";
		}	
	},
	destroy: function() {
		while(this.events && this.events.length > 0) {
			dojo.disconnect(this.events.pop());
		}
		dojo.disconnect(this._clickEvent);
		this._unregisterUpdateEvents();
		this.inherited(arguments);
	},
	close: function(event) {
		//call superclass
		var keepClosing = this.inherited(arguments);
		if (keepClosing === false) return;
		
		if (this._lastFocus) dijit.focus(this._lastFocus);

		// remove the blocker div
		if (this.blockerDiv && this.blockerDiv.parentNode) {
			this._blockerDivAnim = dojo.animateProperty({node: this.blockerDiv, duration: 200, 
				properties: {opacity: {start: BLOCKER_OPACITY, end: 0}},
				onEnd: dojo.hitch(this, this._cleanupBlockerDiv)
			});
			this._blockerDivAnim.play(); 
		}
		this._blockerDivTimeout = setTimeout(dojo.hitch(this,this._cleanupBlockerDiv),1200);
	},
	_cleanupBlockerDiv: function(anim) {
		clearTimeout(this._blockerDivTimeout);
		this._blockerDivTimeout = null;
		if (this._blockerDivAnim) {
			this._blockerDivAnim.stop();
			this._blockerDivAnim = null;
		}
		if (this.blockerDiv) {
			this.blockerDiv.parentNode.removeChild(this.blockerDiv);
			this.blockerDiv = null;
		}
	},
	_onClose: function() {
		while(this.events && this.events.length > 0) {
			dojo.disconnect(this.events.pop());
		}
		this._unregisterUpdateEvents();
		this.inherited(arguments);
	}
});
})();
