/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.client.internal.AdminClient");

dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.team.repository.web.client.init.InitializationData");
dojo.require("jazz.app.auth");
dojo.require("jazz.client.xhr");


(function() {

var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var InitializationData     = com.ibm.team.repository.web.client.init.InitializationData;

// methods
var GET_ALL_ROLES_METHOD = "getAllRoles";
var GET_CONTRIBUTORS_METHOD = "getContributors";
var GET_USER_REGISTRY_INFO_METHOD = "getUserRegistryInfo";
var GET_CONTRIBUTOR_BY_UUID_METHOD = "getContributorByUUID";
var POST_CONTRIBUTOR_METHOD = "postContributor";
var POST_MULTIPLE_NEW_CONTRIBUTORS_METHOD = "postMultipleNewContributors";
var POST_USER_PASSWORD_METHOD = "postUserPassword";
var POST_ARCHIVE_USER_METHOD = "postArchiveUser";
var POST_REMOVE_ADMIN_ROLES_METHOD ="postRemoveAdminRoles";
var X_AUTH_MSG = "X-com-ibm-team-repository-web-auth-msg";
var FORM_SESSION_COOKIE = "JazzFormAuth";

/**
 * Provides helper methods to fetch data from the repository package.
 */
com.ibm.team.repository.web.client.internal.AdminClient= {
	
	// service URI
	SERVICE_URI: "com.ibm.team.repository.service.internal.IAdminRestService",

	postContributor: function (args, responseHandler) {
		AdminClient._invokeRepositoryService(responseHandler, POST_CONTRIBUTOR_METHOD, args);
	},
	
	postMultipleNewContributors: function (args, responseHandler) {
		AdminClient._invokeRepositoryService(responseHandler, POST_MULTIPLE_NEW_CONTRIBUTORS_METHOD, args);
	},
	
	postArchiveUser: function (args, responseHandler) {
		AdminClient._invokeRepositoryService(responseHandler, POST_ARCHIVE_USER_METHOD, args);
	},
	
	postRemoveAdminRoles: function (responseHandler) {
		AdminClient._invokeRepositoryService(responseHandler, POST_REMOVE_ADMIN_ROLES_METHOD);
	},
	
	getAllRoles: function (responseHandler) {
		AdminClient._invokeRepositoryService(responseHandler, GET_ALL_ROLES_METHOD);
	},
	
	getContributorByUUID: function (args, responseHandler) {
		AdminClient._invokeRepositoryService(responseHandler, GET_CONTRIBUTOR_BY_UUID_METHOD, args);
	},
	
	changeUserPassword: function (args, responseHandler) {
		AdminClient._invokeRepositoryService(responseHandler, POST_USER_PASSWORD_METHOD, args);
	},
	
	getUserRegistryInfo: function (responseHandler) {
		AdminClient._invokeRepositoryService(responseHandler, GET_USER_REGISTRY_INFO_METHOD);
	},
	
	getContributors: function (args, responseHandler) {
		if (!args.isLicenseSearch) {
			if (!args.searchTerm) {
				args.searchTerm = "%";
			}
			else {
				args.searchTerm = "%" + args.searchTerm + "%";
			}
		}
		if (args.pageSize == null) {
			args.pageSize = 50;
		}
		if (args.pageNum == null) {
			args.pageNum = 0;
		}
		if (args.filterOutUsers != null && args.filterOutUsers.length > 0) {
			args.filterOutUsers = dojo.toJson(args.filterOutUsers);		
		}
		AdminClient._invokeRepositoryService(responseHandler, GET_CONTRIBUTORS_METHOD, args);
	},
			
	_invokeRepositoryService: function(responseHandler, method, appArgs) {
		// appArgs is optional, so test whether or not it was sent
		if (arguments.length == 2) {
			var serviceRequest= new com.ibm.team.repository.web.transport.ServiceRequest(
					AdminClient.SERVICE_URI,
					method);
		} else {
			serviceRequest= new com.ibm.team.repository.web.transport.ServiceRequest(
					AdminClient.SERVICE_URI,
					method,
					appArgs);
		}
		com.ibm.team.repository.web.transport.TeamServerClient.invokeService(
				serviceRequest,
				responseHandler);
	},
			
	getSimpleLogin: function (responseHandler) {	
		var xhrArgs= {
	        url: net.jazz.ajax._contextRoot + "/auth/whoami",
	        content: { },
			handleAs: "json",
			headers: { },
			handle: dojo.hitch(this, function(response, ioArgs){
				if (response instanceof Error) {
					var errorObj= {
						status: ioArgs.xhr.status
					};
					// format detailed message (unescape URI-encoding formatting, remove leading dashes, and trim whitespace)
					errorObj.message= dojo.trim(unescape(ioArgs.xhr.statusText).replace(/\+/g, " ").replace(/^(-*)/g, ""));
					responseHandler.doErrorCallback(errorObj);
				}
				else {
					responseHandler.doCallback(ioArgs.xhr.getResponseHeader(X_AUTH_MSG));
				}
	        })
	    };
	    jazz.client.xhrGet(xhrArgs);
	}, 
	
	getUserRoles: function (responseHandler) {	
		var xhrArgs= {
	        url: net.jazz.ajax._contextRoot + "/authenticated/identity",
	        content: { },
			handleAs: "json",
			headers: { },
			handle: dojo.hitch(this, function(response, ioArgs){
				if (response instanceof Error) {
					var errorObj= {
						status: ioArgs.xhr.status
					};
					// format detailed message (unescape URI-encoding formatting, remove leading dashes, and trim whitespace)
					errorObj.message= dojo.trim(unescape(ioArgs.xhr.statusText).replace(/\+/g, " ").replace(/^(-*)/g, ""));
					responseHandler.doErrorCallback(errorObj);
				}
				else {
					responseHandler.doCallback(response);
				}
	        })
	    };
	    jazz.client.xhrGet(xhrArgs);
	} 
};

var AdminClient= com.ibm.team.repository.web.client.internal.AdminClient;

})();
