/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("net.jazz.ajax.core.internal.Uri");

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");

(function() {

var checkNotNull 	= net.jazz.ajax.toolbox.lang.checkNotNull;
var checkNotEmpty 	= net.jazz.ajax.toolbox.lang.checkNotEmpty;

function replaceAfterToken(string, token, replacement) {
	checkNotNull(token, "token");
	checkNotNull(string, "string");
	checkNotNull(replacement, "replacement");
	
	if(string.indexOf(token) == -1) {
		return string + token + replacement;
	}
	
	return string.slice(0, string.indexOf(token) + 1) + replacement;
}

function setUriFragment(uri, fragmentString) {
	checkNotNull(uri, "uri");
	checkNotNull(fragmentString, "fragmentString");
	
	return replaceAfterToken(uri, "#", fragmentString);
}

net.jazz.ajax.core.internal.setUriFragment = setUriFragment;

function setUriQuery(uri, queryString) {
	checkNotNull(uri, "uri");
	checkNotNull(queryString, "queryString");
	
	return replaceAfterToken(uri, "?", queryString);
}

net.jazz.ajax.core.internal.setUriQuery = setUriQuery;

function getUriFragmentStatements(uri) {
	checkNotNull(uri, "uri");
	checkNotNull(uri.fragment, "uri.fragment");
	
	return uri.fragment.split("&");
}

function getUriFragmentHash(uri) {
	checkNotNull(uri, "uri");

	if(!uri.fragment) {
		return {};
	}

	var fragmentHash = {};

	dojo.forEach( getUriFragmentStatements(uri), function(x) {
		var queryTuple = x.split("=");

		if (queryTuple.length == 1) { return; }
		
		var key = queryTuple[0];
		var value = decodeURIComponent(queryTuple[1].replace(/\+/g, " "));
		var curValue = fragmentHash[key];

		if (!curValue) {
			// if the new value isn't already in the hash, just add it
			fragmentHash[key] = value;
		} else {
			// otherwise if the value exists as a non-array, convert the
			// value to a single-element array
			if (!dojo.isArray(curValue)) {
				fragmentHash[key] = [curValue];
			}
			// now we can just append the new value to the array
			fragmentHash[key].push(value);
		}
	});
	
	return fragmentHash;
}

net.jazz.ajax.core.internal.getUriFragmentHash = getUriFragmentHash;

})();