/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.util.internal.theming");

dojo.require("jazz.app.proxy");

(function () {
var X_JAZZ_WEB_THEMING_OAUTH_URL = "X-jazz-web-theming-oauth-url";
jazz.util.internal.theming = {
	_loadThemingCSS: function () {
		var theme = jazz.util.internal.theming._getPreviewOrCurrent();
		var styleUrl = net.jazz.ajax._contextRoot + "/_theming/allThemeCSS/" + theme;
		if (document.createStyleSheet === undefined) {
			var styleElement = document.createElement("link");
			styleElement.rel = "stylesheet";
			styleElement.type = "text/css";
			styleElement.href = styleUrl,
			document.getElementsByTagName("head")[0].appendChild(styleElement);
		} else {
			var styleSheet = null;
			if (document.styleSheets.length > 31) {
				for (var i=0; i < document.styleSheets.length; i++) {
					if (djConfig.isDebug) {
						console.debug("stylesheet [" + i + "] has " +document.styleSheets[i].imports.length + " imports");
					}
					if (document.styleSheets[i].imports.length < 32) {
						styleSheet = document.styleSheets[i];
						break;
					}
				}
				if (styleSheet != null) {
					styleSheet.addImport(styleUrl);
				} else {
					throw new Error("No stylesheet is available to add an import to");
				}
			} else {
				styleSheet = document.createStyleSheet(styleUrl);
				if (djConfig.isDebug) { console.debug("Created new stylesheet. Number of current stylesheets is "+document.styleSheets.length); }
			}
		}
	},
	
	_loadThemingJS: function () {
		var dfd = jazz.util.internal.theming._getThemeJS();
		var jsDfd = new dojo.Deferred();
		dfd.addCallback(function (response) {
			if (response) {
				// override dojo.provide and dojo.declare in order to prevent theming JavaScript
				// code from overriding existing JavaScript modules.
				overrideProvideAndDeclare();
				try {
					eval(response);
					resetProvideAndDeclare();
					jsDfd.callback();
				} catch (e) {
					console.error("Unable to load theme JavaScript: " + e);
					resetProvideAndDeclare();
					jsDfd.errback();
				}	
			} else if (response === "") {
				jsDfd.callback();
			} else {
				jsDfd.errback();
			}
		});
		dfd.addErrback(dojo.hitch(this, function (curDfd, response) {
			var header = curDfd.ioArgs.xhr.getResponseHeader(X_JAZZ_WEB_THEMING_OAUTH_URL);
			if (header) {
				var oauthDfd = jazz.client.xhrGet({
					'url': header
				});
			}
			jsDfd.errback();
		}, dfd));
		return jsDfd;
	},
	
	_getThemeJS: function () {
		return jazz.client.xhrGet({
			'url': net.jazz.ajax._contextRoot + "/_theming/allThemeJS/" + jazz.util.internal.theming._getPreviewOrCurrent()
		})
	},
	
	_getPreviewOrCurrent: function () {
		var previewTheme = document.location.href.match("previewTheme=[^&#]*");
		return previewTheme ? previewTheme[0].split("=")[1] : "_current";
	}
};

function pathIsFunction(path) {
	var pathAry = arguments[0].split(".");
	var curObj = window;
	var exists = true;
	for (var i=0; i < pathAry.length; i++) {
		if (curObj[pathAry[i]] == null) {
			exists = false;
			break;
		} else {
			curObj = curObj[pathAry[i]];
		}
	}
	return exists ? dojo.isFunction(curObj) : false;
}

function overrideProvideAndDeclare() {
	jazz.util.internal.theming._dojoProvide = dojo.provide;
	jazz.util.internal.theming._dojoDeclare = dojo.declare;
	dojo.provide = function () {
		if (pathIsFunction(arguments[0])) {
			console.error("Illegal attempt to provide " + arguments[0] + " in theming code");
		} else {
			jazz.util.internal.theming._dojoProvide(this, arguments);
		}
	}
	jazz.util.internal.theming._dojoCustomDeclare = function () {
		if (pathIsFunction(arguments[0])) {
			console.error("Illegal attempt to declare " + arguments[0] + " in theming code");
		} else {
			dojo.declare = jazz.util.internal.theming._dojoDeclare;
			dojo.declare.apply(this, arguments);
			dojo.declare = jazz.util.internal.theming._dojoCustomDeclare;
		}
	}
	dojo.declare = jazz.util.internal.theming._dojoCustomDeclare;
}

function resetProvideAndDeclare() {
	dojo.declare = jazz.util.internal.theming._dojoDeclare;
	dojo.provide = jazz.util.internal.theming._dojoProvide;
}
})();
