/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("net.jazz.ajax.ui.PageContainer");

dojo.require("dijit._Widget");

dojo.require("net.jazz.ajax.core.internal.util");
dojo.require("net.jazz.ajax.ui.ErrorPage");
dojo.require("jazz.util.ResizeNotifier");

dojo.declare("net.jazz.ajax.ui.PageContainer", dijit._Widget, {
	
	// summary:
	//		a widget for displaying pages
	// usage notes:
	//		this widget contains no public methods; simply instantiate it and
	//		place it within a Jazz Framework-based application
	// since:
	//		0.6
	
	//
	// Internal and lifecycle methods; do not call directly
	//
	
	create: function(args, srcNodeRef) {
		if(!srcNodeRef) {
			srcNodeRef = document.createElement("div");
		}
		// subscribe to events
		
		net.jazz.ajax.ui.PageContainer.superclass.create.call(this, args, srcNodeRef);
	},
	
	postCreate: function() {
		this.domNode.id = "net-jazz-ajax-PageContainer";
		
		// subscribe to relevant events
		dojo.subscribe(net.jazz.ajax.internal.events.viewPage, this, "_viewPage");
		dojo.subscribe(net.jazz.ajax.internal.events.viewErrorPage, this, "_viewErrorPage");
	},
	
	_viewPage: function(page) {
		// if we're already viewing the incoming page, bail out
		if(this._currentPage && this._currentPage === page.domNode) {
			return;
		}
		
		// otherwise display the page
		if(this._currentPage) {
			this.domNode.replaceChild(page.domNode, this._currentPage);
		} else {
			this.domNode.appendChild(page.domNode);
		}
		
		this._currentPage = page.domNode;
		jazz.util.ResizeNotifier.poke();
	},
	
	_viewErrorPage: function(message, finePrint, hideRunDefaultAction) {
		if (this._errorPage) {
			this._errorPage.setHideRunDefaultAction(hideRunDefaultAction);
		} else {
			this._errorPage = new net.jazz.ajax.ui.ErrorPage({hideRunDefaultAction: hideRunDefaultAction});
		} 
		this._errorPage.setMessage(message, finePrint);
		
		if (this._currentPage == this._errorPage.domNode) return;
		
		if(this._currentPage)
			this.domNode.replaceChild(this._errorPage.domNode, this._currentPage);
		else
			this.domNode.appendChild(this._errorPage.domNode);
		this._currentPage = this._errorPage.domNode;
	}
});