/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.FileUploader");

dojo.require("dojo.io.iframe");

dojo.require("net.jazz.ajax.core.internal.util");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "FileUploaderMessages");


var callContributedCode = net.jazz.ajax.core.internal.util.callContributedCode;

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.FileUploader", [dijit._Widget, dijit._Templated], {
	templateString: "<form method=\"post\" enctype=\"multipart/form-data\" style=\"padding-bottom: 10px;\"></form>",
	
	constructor: function(args) {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "FileUploaderMessages");
		this.postURI = checkParam(args.postURI, this.messages.MSG_MISSING_PARAM + "postURI");
		this.onUpload = checkParam(args.onUpload, this.messages.MSG_MISSING_PARAM + "onUpload");
		this.onUploadFailure = checkParam(args.onUploadFailure, this.messages.MSG_MISSING_PARAM + "onUploadFailure");
		this.formElements = args.formElements || [];
		this.formLabel = args.formLabel;
	},
	
	postCreate: function() {
		this._resetFileInput();
	},
	
	_onFileInputChange: function(e) {
		if(dojo.isFunction(this.onBeginUpload)) {
			this.onBeginUpload();
		}
		
		// start upload request
		dojo.io.iframe.send({
			form: this.domNode,
			url: this.postURI,
			handleAs: "json",
			handle: dojo.hitch(this, this._onUploadResponse),
			timeout: 45000
		});
	},
	
	_onUploadResponse: function(responseObj, ioArgs) {
		this._resetFileInput();
		var self = this;
		callContributedCode( function() {
			if(responseObj instanceof Error && responseObj.dojoType === "timeout") {
				responseObj.errorMessage = self.messages.TIMEOUT_UNAVAILBLE_MSG;
			} else if (dojo.isIE && responseObj instanceof Error && responseObj.name === "TypeError") {
				responseObj.errorMessage = self.messages.IE_SERVER_UNAVAILBLE_MESSAGE;
			}
			if (responseObj.errorMessage) {
				self.onUploadFailure(responseObj);
			} else {
				self.onUpload(responseObj)
			}
		});
	},
	
	_resetFileInput: function() {
		if(this.fileInput){
			this.disconnect(this._onChangeEvent);
			removeChildren(this.domNode);
		}
		
		if (this.formLabel) {
			this.domNode.appendChild(document.createTextNode(this.formLabel));
		}
		this.fileInput = document.createElement("input");
		this.fileInput.type = "file";
		this.fileInput.name = "fileInput";
		this.domNode.appendChild(this.fileInput);
		this.inline = document.createElement("div");
		this.domNode.appendChild(this.inline);
		var self = this;
		dojo.forEach(this.formElements, function(formElement) {
			var input = document.createElement("input");
			input.type = "hidden";
			input.name = formElement.name;
			input.value = formElement.value;
			self.domNode.appendChild(input);
		});
		this._onChangeEvent = this.connect(this.fileInput, "onchange", this._onFileInputChange);
	}, 
	
	setInlineMessage: function (message, className) {
		this.inline.innerHTML = message;
		this.inline.className = className ? className : "";
	}
});


function checkParam(param, msg) {
	if(!param) {
		throw new Error(msg)
	}
	return param;
}

function removeChildren(node) {
	while(node.firstChild) {
		node.removeChild(node.firstChild);
	}
}

})();