/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
    dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.LicenseEditor");
    
    dojo.require("dojo.date");
    dojo.require("dojo.date.locale");
    dojo.require("dojo.date.stamp");
    dojo.require("dojo.number");
    dojo.require("dijit._Widget");
    dojo.require("dijit._Templated");
    dojo.require("dojox.data.dom");
    
    dojo.require("jazz.ui.HoverPopup");
    dojo.require("net.jazz.ajax.ui.PlatformUI");
    
    dojo.require("com.ibm.team.repository.web.client.internal.licensing");
    dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
	dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
    dojo.require("com.ibm.team.repository.web.ui.internal.Util");

    dojo.require("dojo.i18n");
    dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "LicenseEditorMessages");
    
	dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.LicenseHelper");
    dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.LicenseWizard");
    dojo.require("com.ibm.team.repository.web.ui.wizard.Wizard");
    dojo.require("com.ibm.team.repository.web.ui.wizard.WizardStep");
    dojo.require("com.ibm.team.repository.web.ui.wizard.WizardStepHeader");
	dojo.require("com.ibm.team.repository.web.ui.wizard.WizardConfigurationStep");
	dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.LicenseConfigFloatingStep");	
    dojo.require("jazz.ui.StyledBox");
    dojo.require("jazz.ui.Dialog");
	dojo.require("jazz.ui.Button");
    
    //
    // Aliases
    //
    
    var LIC_P = com.ibm.team.repository.web.client.internal.licensing;
    var formatCALTypeDescription = LIC_P.formatCALTypeDescription;
    var getLicenseInfo = LIC_P.getLicenseInfo;
    var getUsedLicenseInfo = LIC_P.getUsedLicenseInfo;
    var getExpirationInfo = LIC_P.getExpirationInfo;
    var sortCALs = LIC_P.sortCALs;
    var HoverPopup = jazz.ui.HoverPopup;
    var removeChildren = dojox.data.dom.removeChildren;
    var SRH = com.ibm.team.repository.web.transport.ServiceResponseHandler;
    var LicenseHelper = com.ibm.team.repository.web.ui.internal.admin.server.LicenseHelper;
	var LicenseWizard = com.ibm.team.repository.web.ui.internal.admin.server.LicenseWizard;
    var FLOATING_SUPPORT_OPERATION_ID = "com.ibm.team.repository.support.floatingLicenses";
    var ACTION_ID = "action=com.ibm.team.repository.admin.manageLicenses";
    var WizardStepHeader = com.ibm.team.repository.web.ui.wizard.WizardStepHeader;
    var WizardStep = com.ibm.team.repository.web.ui.wizard.WizardStep;
	var WizardConfigurationStep = com.ibm.team.repository.web.ui.wizard.WizardConfigurationStep;
	var FloatingConfigStep = com.ibm.team.repository.web.ui.internal.admin.server.LicenseConfigFloatingStep;
	var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
	var DEFAULT_LICENSE_SERVICE = "com.ibm.team.repository.service.internal.license.DefaultLicenseProviderService";
	
    var messages;
    //
    // Widget definition
    //
    
    dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.LicenseEditor", [LicenseHelper, dijit._Widget, dijit._Templated], {
    
        //
        // Class properties
        //
        
        templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/LicenseEditor.html"),
        
        //
        // Widget lifecycle methods
        //
        constructor: function(){
            this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "LicenseEditorMessages");

            var detailsLink = "<a href='#action=com.ibm.team.repository.admin.manageLicenses'>";
            this.clientDescriptionIncludingLink = dojo.string.substitute(this.messages.clientDescription, [detailsLink, "</a>"]);
            this.noFloatingSupportIncludingLink = dojo.string.substitute(this.messages.noFloatingSupport, [detailsLink, "</a>"]);
            this.floatingUploadWarningIncludingLink = dojo.string.substitute(this.messages.floatingUploadWarning, [detailsLink, "</a>"]);
            
        },
        
        postCreate: function(){
        
            this._setInfopopHeading(this.totalHeader, "totalHeader", "totalHeaderInfo");
            this._setInfopopHeading(this.assignedHeader, "assignedHeader", "assignedHeaderInfo");
            this._setInfopopHeading(this.availableHeader, "availableHeader", "availableHeaderInfo");
            this._setInfopopHeading(this.floatTotalHeader, "floatTotalHeader", "floatTotalHeaderInfo");
            
			this.header = new WizardStepHeader({
	        	stepTitle: this.messages.heading,
	            reset: dojo.hitch(this,"reload")
	        });
	        this.headerArea.appendChild(this.header.domNode);
            this.reload();
        },
		
		launchFloatingConfig: function () {
			
			if (!this.licenseInfo) {
				//License info request failed. don't launch wizard
				return;
			} else {
			
				var self = this;
				this.wiz = new LicenseWizard({
					licenseInfo: self.licenseInfo,
					parentPointer: self
				});
				
				var uriConfigArgs = {
					stepTitle: this.messages.configFloatingStepTitle,
					parentPointer: this,
					description: this.messages.configFloatingdescription,
					helpContent: this.messages.configFloatingHelp,
					implementationClassName: "com.ibm.team.repository.service.internal.license.LicenseService",
					extraServiceConfigArgs: {
						hiddenPropertyList: ["licenseService.floatingLicenseRefreshRate"],
						forceSimple: true
					},
					innerArgs: {
						headingLabel: this.messages.configFloatingBlockLabel,
						heading: ""
					},
					canFinish: function(){
						return this.enableNext === true;
					}
				}
				this.uriConfigStep = new FloatingConfigStep(uriConfigArgs);
				
				var wizardArgs = {
					steps: [this.uriConfigStep],
					width: "870px",
					primaryTitle: this.messages.configFloatingStepTitle,
					showStepSummary: false,
					processFinish: function(){
						this._steps[0].invokeSave(self, "saveLicenseUpdates", "saveLicenseError");
					},
					processCancel: function(){
						var verifyCancel = confirm(self.messages.cancelWizard);
						if (verifyCancel) {
							this.container.close();
						}
					}
				};
				this.wizard = new com.ibm.team.repository.web.ui.wizard.Wizard(wizardArgs);
			}

		},
        
        launchWizard: function() {
        
            var self = this;
			
			if (!this.licenseInfo) {
				//License info request failed. don't launch wizard
				return;
			} else {
			
				//Step One: License Uploads
				this.wiz = new LicenseWizard({
					mode: "upload",
					licenseInfo: self.licenseInfo,
					parentPointer: self,
					stepTitle: this.messages.step1Title
				});
				
				var wizardArgs = {
					steps: [this.wiz],
					width: "870px",
					primaryTitle: this.messages.step1Title,
					showStepSummary: true,
					processFinish: function(){
						var numSteps = self.wizard.contentWidget._steps.length;
						//Don't need to save Floating Connection if they never upload a floating CAL!
						if (numSteps == 1) {
							self.wiz.saveEditor();
						}
						else {
							self.wiz.saveFloatingConnection();
						}
						
					},
					processCancel: function(){
						var verifyCancel = confirm(self.messages.cancelWizard);
						if (verifyCancel) {
							this.container.close();
						}
					}
				};
				this.wizard = new com.ibm.team.repository.web.ui.wizard.Wizard(wizardArgs);
			}
        },

		saveLicenseUpdates: function (){
			this.wiz.saveEditor();
		},
		
		saveLicenseError: function (errObj) {
			this.wiz.cbSaveError(errObj);
		},
		
		launchFloatingEditor: function() {
			//passthru to the "real" CAL editor in order to control CALs displayed
			var floatingOn = true;
			this.launchCALEditor(floatingOn);
		},
		launchCALEditor: function(showFloating) {
			if (!this.licenseInfo) {
				//License info request failed. don't launch wizard
				return;
			} else {
			
				var self = this;		
				
				//If this function is called directly, showFloating is a click event
				if (showFloating != true) 
					showFloating = false;
					
				var sTitle = showFloating ? this.messages.editFloatTitle : this.messages.editCALTitle;
				
				//Step One: Change CAL purchase counts
				this.editor = new LicenseWizard({
					mode: "editCALs",
					floatingOn: showFloating,
					licenseInfo: self.licenseInfo,
					parentPointer: self,
					stepTitle: sTitle
				});
				
				var wizardArgs = {
					steps: [this.editor],
					width: "870px",
					primaryTitle: sTitle,
					showStepSummary: false,
					processFinish: function(){
						self.editor.saveEditor();
					},
					processCancel: function(){
						var verifyCancel = confirm(self.messages.cancelWizard);
						if (verifyCancel) {
							this.container.close();
						}
					}
					
				};
				this.wizard = new com.ibm.team.repository.web.ui.wizard.Wizard(wizardArgs);
			}
		},
		
		launchDefaultWizard: function() {
			if (!this.licenseInfo) {
				//License info request failed. don't launch wizard
				return;
			} else {
			
				var self = this;		
				var sTitle = this.messages.defaultWizardHeader;
				
				this.setDefault = new LicenseWizard({
					mode: "setDefault",
					licenseInfo: self.licenseInfo,
					defaultLicense: self.defaultLicense,
					parentPointer: self,
					stepTitle: sTitle
				});
				
				var wizardArgs = {
					steps: [this.setDefault],
					width: "870px",
					primaryTitle: sTitle,
					showStepSummary: false,
					processFinish: function(){
						self.setDefault.saveDefaultLicense();
					},
					processCancel: function(){
						var verifyCancel = self.setDefault.hasDefaultChanged() ? confirm(self.messages.cancelWizard) : true;
						if (verifyCancel) {
							this.container.close();
						}
					}		
				};
				this.wizard = new com.ibm.team.repository.web.ui.wizard.Wizard(wizardArgs);
			}
		},
        
        /**
         * Fetches the most recent saved license information from the server and
         * resets the editor with this information (long-op).
         */
        reload: function(){           
            var self = this;
            var o = {
                s: function(licenseInfo){
                    self.header.clearMessages();
                    self.reset(licenseInfo);
                    self.initialLicenseInfo = self.licenseInfo = licenseInfo;                    
                },
                f: function(err){
                    self.displayServiceCallbackErrorMessage(err);
                }
            }
            getLicenseInfo(new SRH(o, "s", "f"));
        },
        
        rebind: function(){
			this.refreshServerTable(this.serverInfoTable);
            this.refreshCALTable();
            this.refreshFloatingCALTable();
        },
        
        reset: function(licenseInfo){
            
            this.initialLicenseInfo = this.licenseInfo = licenseInfo;
            this.usedLicenseInfo = null;
            this.lastFloatingCount = licenseInfo.floatingLicenseTypes && licenseInfo.floatingLicenseTypes.length ? licenseInfo.floatingLicenseTypes.length : 0;
            this.rebind();
            
            var self = this;
            var floatHelper = {
                yes: function(status){
                    self.addLicenseUsageDetails();
					self.AllowFloatingEdit.style.display = "inline";
                },
                no: function(s){
                    licenseInfo
                    self.usedLicenseInfo = null;
					self.AllowFloatingEdit.style.display = "none";
                },
                error: function(err){
                    self.displayServiceCallbackErrorMessage(err);
                }
            }
            
            LIC_P.checkFloatingLicensesSupported(floatHelper.yes, floatHelper.no, floatHelper.error);
        },
		
		displayServiceCallbackErrorMessage: function(errorObj) {
	        this.header.displayServiceCallbackErrorMessage(errorObj);
	    },
 
        /**
         * Fetches the detailed breakdown of the license usage and updates the UI.
         * This method will only success if floating licenses are supported on the server
         */
        addLicenseUsageDetails: function(){
            this.licenseInfo.operationCheckResults = [{
                "operationId": FLOATING_SUPPORT_OPERATION_ID,
                "allowed": true
            }];
            var self = this;
            var floatingLicenseInfoHandler = {
                success: function(info){
                    self.updateUsedLicenseData(info);
                },
                failure: function(err){
                    self.usedLicenseInfo = null;
                    self.displayServiceCallbackErrorMessage(err);
                }
            }
            getUsedLicenseInfo(new SRH(floatingLicenseInfoHandler, "success", "failure"));
        },
        
        /**
         * This is called when addition details of the used
         * licenses on the server.  It provides a breakdown
         * between floating and non-floating use.  This method
         * might not be called if the underlying server does not
         * support floating licenses
         
         */
        updateUsedLicenseData: function(usedLicenseInfo){
            this.usedLicenseInfo = usedLicenseInfo;
            
            if (this.licenseInfo) {
				this.refreshServerTable(this.serverInfoTable);
			}
                
        },
        
        
        /**
         * Refreshes the Client Access License table based on the current value of
         * <code>this.licenseData.types</code>.
         */
        refreshCALTable: function(){
			
			//Only show the "Expired On" header if some CALs have expiration
			var hasExpiringCALs = LIC_P.checkExpiringCALs(this.licenseInfo, false);
			this.CALExpiresHeader.style.display = hasExpiringCALs ? "block" : "none";
            
            var tbody = this.CALInfoTable;
            var self = this;
            var inputs = this.CALInfoTable.getElementsByTagName("input");
            dojo.forEach(inputs, function(input){
                if (input.__jazzChgEvt) {
                    self.disconnect(input.__jazzChgEvt);
                }
            });
            removeChildren(tbody);
            this.EditButtonCAL.style.display = "none";
            // normalize data and create table columns
            var types = this.licenseInfo.types || [];
            types = sortCALs(types);
            var isTrial = this.licenseInfo.serverLicense.expirationTime ? true : false;
			var count = 0;
			dojo.forEach(types, function(type){
				count++;
				self.EditButtonCAL.style.display = "inline";
				
				var icon = d.ce("div");
				var status = d.ce("span");
				
                var row = d.ce("tr");
		 		tbody.appendChild(row);
                // deal with disabled CALs
				
				var nameLink = self._getInfopopLink(type.name, formatCALTypeDescription(type));
				var isDisabled = type.disabledReason != null;
				
				if (isDisabled) {
                    row.className = "disabledCAL";
					icon.className = "icon-error";
					dojo.addClass(nameLink,"disabledCAL");
					status.innerHTML = dojo.string.substitute(self.messages.deactivatedReason, [type.disabledReason]);
					dojo.addClass(status,"WARNINGcolor");
                } else {
					icon.className = "icon-ok";
					status.innerHTML = self.messages.activated;
				}
                
                var limit = getCountOrUnlimited(type.max);
                var included = getCountOrUnlimited(type.free);
                var total = getCountOrUnlimited(type.total);
                
                // Set displayable counts depending on whether the product is a
                // Permanent edition or a trial
                if (isTrial) {
					var available = type.max === -1 ? self.messages.unlimited : type.max - type.used;
					var purchase = type.max === -1 ? self.messages.notApplicable : type.max - type.free;
					var total = getCountOrUnlimited(type.max);
				}
				else {
					var available = type.total === -1 ? self.messages.unlimited : type.total - type.used;
					var purchase = type.free === -1 ? self.messages.notApplicable : type.total - type.free;
				}				
				
                // Create the columns for the given CAL
                var nodes = [type.productName, type.editionName + " (" + type.variantName + ")", nameLink, 
								total, getUsedLink(type), available, icon, status];
								
				if(hasExpiringCALs) {
					//Get CAL expiration info
					var expiresOn = type.expirationTime;
					if(expiresOn != null) {
						var expirationInfo = getExpirationInfo(expiresOn);
						expiresOn = expirationInfo.expirationDateString;
					} else {
						expiresOn = "";
					}
					nodes.push(expiresOn);
				}
				
				
                dojo.forEach(nodes, function(node){
                    col(row, node);
                });
            });
			if(count == 0) {
				//No CALs uploaded so far, show message.
				var tr = d.ce("tr");
				var td = d.ce("td");
				td.className = "noRowsYet";
				td.colSpan = 8;
				td.innerHTML = this.messages.noCALsYet;
				tr.appendChild(td);
				tbody.appendChild(tr);
			}
			
        },
        
        
        /**
         * Refreshes the Floating Client Access License table based on the current value of
         * <code>this.licenseData.floatingLicenseTypes</code>.
         */
        refreshFloatingCALTable: function(){
			
			//Only show the "Expired On" header if some CALs have expiration
			var hasExpiringCALs = LIC_P.checkExpiringCALs(this.licenseInfo, true);
			this.FloatExpiresHeader.style.display = hasExpiringCALs ? "block" : "none";
			
            var tbody = this.FloatCALInfoTable;
            var self = this;
            removeChildren(tbody);
            this.EditButtonFloat.style.display = "none";
            // normalize data and create table columns
            var types = this.licenseInfo.floatingLicenseTypes || [];
   //         dojo.toggleClass(this.FloatCALInfo, "noDisplay", !this.licenseInfo.floatingLicenseTypes);
            if (this.licenseInfo.floatingLicenseTypes) {
                types = sortCALs(types);
                dojo.forEach(types, function(type){
                    //Enable the EDIT button
					self.EditButtonFloat.style.display = "inline";

					var icon = d.ce("div");
					var status = d.ce("span");
					
	                var row = d.ce("tr");
			 		tbody.appendChild(row);
					
					var nameLink = self._getInfopopLink(type.name, formatCALTypeDescription(type));
					var isDisabled = type.disabledReason != null;
					
	                if (isDisabled) {
	                    row.className = "disabledCAL";
						icon.className = "icon-error";
						dojo.addClass(nameLink,"disabledCAL");
						status.innerHTML = dojo.string.substitute(self.messages.deactivatedReason, [type.disabledReason]);
						dojo.addClass(status,"WARNINGcolor");
	                } else {
						icon.className = "icon-ok";
						status.innerHTML = self.messages.activated;
					}				
                    
                    // Trial mode of server has no effect on counts for floating
                    var limit = getCountOrUnlimited(type.max);
                    var included = getCountOrUnlimited(type.free);
                    var total = getCountOrUnlimited(type.total);
                    var purchase = type.free === -1 ? self.messages.notApplicable : type.total - type.free;
                    
                    var nodes = [type.productName, type.editionName + " (" + type.variantName + ")", nameLink, total,getIssuedLink(type),icon, status];
                   
				    if(hasExpiringCALs) {
						//Get CAL expiration info
						var expiresOn = type.expirationTime;
						if(expiresOn != null) {
							var expirationInfo = getExpirationInfo(expiresOn);
							expiresOn = expirationInfo.expirationDateString;
						} else {
							expiresOn = "";
						}
						nodes.push(expiresOn);
					}
				   
				   
				    dojo.forEach(nodes, function(node){
                        col(row, node);
                    });
                });
            } else {
			
				//No CALs uploaded so far, show message.
				var tr = d.ce("tr");
				var td = d.ce("td");
				td.className = "noRowsYet";
				td.colSpan = 8;
				td.innerHTML = this.messages.noCALsYet;
				tr.appendChild(td);
				tbody.appendChild(tr);
			}
        },
        
        /**
         * Refreshes the Server License information table based on the current value
         * of <code>this.licenseData.serverLicense</code>.
         */
        refreshServerTable: function(tbody,licInfo){
			var serverLicense = licInfo ? licInfo : this.licenseInfo.serverLicense;
            removeChildren(tbody);
            var isTrial = serverLicense.expirationTime ? true : false;
            if (isTrial) {
                var expirationInfo = getExpirationInfo(serverLicense.expirationTime);
            }
            var isDisabled = serverLicense.disabledReason || (isTrial && expirationInfo.isExpired);
            function row(propRow, className){
                var row = d.ce("tr");
                row.className = isDisabled ? "disabledServer" : "";
                tbody.appendChild(row);
                var td = d.ce("td");
                row.appendChild(td);
                td.innerHTML = propRow.n;
                td.className = "propColumn";
                var td = d.ce("td");
                row.appendChild(td);
                td.innerHTML = propRow.v;
				if(propRow.id) td.id = propRow.id;
                if (className) {
                    td.className = className;
                }
            }
            
            var maxUsers = serverLicense.maxContributors !== -1 ? serverLicense.maxContributors : this.messages.unlimited;
            var editionLabel = serverLicense.editionName ? serverLicense.editionName : "";
            editionLabel += serverLicense.variantName ? " (" + serverLicense.variantName + ")" : "";
            var usedLicenseText = serverLicense.usedContributors;
            if (this.usedLicenseInfo) {
				var fStatus = this.licenseInfo.floatingSetupStatus;
				if (fStatus.serverURL && fStatus.serverURL != "") {					
	                 var numFloating = this.usedLicenseInfo.countedActiveLeases ? this.usedLicenseInfo.countedActiveLeases : 0;
	                 var activeLabel = dojo.string.substitute(this.messages.acquiredLinkLabel, [ numFloating ]);
	                 var activeFloatingLink = "<a href=\"#action=com.ibm.team.repository.activeFloatingLicenses\">" + activeLabel + "</a>" ;
	                 var assigned = this.usedLicenseInfo.totalUsedCount - numFloating;
	                 var assignedLabel = dojo.string.substitute(this.messages.assignedLinkLabel, [ assigned ]);
	         //      var assignedLink = "<a href=\"#action=com.ibm.team.repository.manageUserCALs\">" + assignedLabel + "</a>" ;
	                 usedLicenseText = this.usedLicenseInfo.totalUsedCount + " (" + assignedLabel + ", " + activeFloatingLink + ")";
	            }
                 
                if (findOperationCheckResults(this.licenseInfo, FLOATING_SUPPORT_OPERATION_ID) == false) {
					var warningSection = "<span class=\"inline-message WARNINGcolor\">" + this.messages.noFloatSupport + "</span>";
					usedLicenseText = usedLicenseText + warningSection;
				}  
            }
            
            dojo.toggleClass(this.floatingAllowedContent, "noDisplay", !this.usedLicenseInfo);
			dojo.toggleClass(this.floatingNotAllowedContent, "noDisplay", this.usedLicenseInfo);
            
            var isFloatingServer = dojo.getObject("licenseInfo.floatingSetupStatus.server", false, this);
            if (this.usedLicenseInfo && isFloatingServer) {
                dojo.publish(com.ibm.team.repository.web.internal.events.floatingLicenseChange, [{
                    actingAsFloating: true
                }]);
            }
            
            var rows = [{
                n: this.messages.serverProduct,
                v: serverLicense.productName
            }, {
                n: this.messages.serverEdition,
                v: editionLabel
            }, {
                n: this.messages.serverDescription,
                v: serverLicense.description
            }, {
                n: this.messages.maximumUsers,
                v: maxUsers
            }, {
                n: this.messages.countedUsers,
                v: usedLicenseText
            }];
            dojo.forEach(rows, function(propRow){
                row(propRow);
            });
            
            var className = null;
            if (serverLicense.disabledReason) {
                var statusString = serverLicense.disabledReason;
            }
            else {
                var statusString = !isTrial ? this.messages.permanent : expirationInfo.isExpired ? this.messages.expiredTrial : this.messages.trial;
            }
            row({
				id: "serverStatusField",
                n: this.messages.serverStatus,
                v: statusString
            }, isDisabled ? "expired" : null);

            if (isTrial) {
				
				if(dojo.byId('serverStatusField')) {
					//Add activate link
					var a1Element = document.createElement("a");
					a1Element.className = "connectLink";
					a1Element.onclick = dojo.hitch(this,"launchWizard");
					a1Element.innerHTML = this.messages.activateText;
					dojo.byId('serverStatusField').appendChild(a1Element);
				}
				
				
                if (expirationInfo.isExpired) {
                    var rowObj = {
                        n: this.messages.expiredOn,
                        v: expirationInfo.expirationDateRelative
                    };
                }
                else {
                    var rowObj = {
                        n: this.messages.expiresOn,
                        v: expirationInfo.expirationDateRelative
                    };
                }
                row(rowObj, expirationInfo.isExpired ? "expired" : null);
            }
			
			//Add property for floating license information
			
			if (!licInfo && this.licenseInfo.floatingSetupStatus) { // i.e. not used for preview table in wizard
			
				var licenseInfo = this.licenseInfo;
				var status;
				var isFServerStatus;
				
	
				if (licenseInfo.floatingSetupStatus.server === true) {
					isFServerStatus = this.messages.configuredServer;
					var n = licenseInfo.floatingLicenseTypes.length;
					//Different message to adjust for singular/plural case
					var msg = (n==1) ? this.messages.configuredServerOne : this.messages.configuredServer;
					isFServerStatus = dojo.string.substitute(msg, [ n ]); 
				} else {
					isFServerStatus = this.messages.notConfiguredServer;
				}	

				if (licenseInfo.floatingSetupStatus.client === true) {
					var connectText = this.messages.modifyLink;
					var licenseServerLink = "<a href=\"" + licenseInfo.floatingSetupStatus.serverURL + "\">" + licenseInfo.floatingSetupStatus.serverURL + "</a>"
					status = dojo.string.substitute(this.messages.configuredClient, [ licenseServerLink ]); 
					if (licenseInfo.floatingSetupStatus.connectionError) {
						status = status + "<br/><span class=\"error\">" + this.messages.licenseServerError + licenseInfo.floatingSetupStatus.connectionError + "</span>";
					}
				} else {
					var connectText = this.messages.connectText;
					status = this.messages.notConfiguredClient; 
				}
				row({
	                id: "floatingClientStatus",
					n: this.messages.floatingServer,
	                v: status
	            });
				
				if(dojo.byId('floatingClientStatus')) {
					//Add connect link
					var aElement = document.createElement("a");
					aElement.className = "connectLink";
					aElement.onclick = dojo.hitch(this,"launchFloatingConfig");
					aElement.innerHTML = connectText;
					dojo.byId('floatingClientStatus').appendChild(aElement);
					
					var divElem = document.createElement("div");
					divElem.innerHTML = isFServerStatus;
					dojo.byId('floatingClientStatus').appendChild(divElem);
				}
			}
				
			//get the default license id and then display it.
			var srh = new SRH(this,"getDefaultLicense","displayServiceCallbackErrorMessage");
			ServerStatusClient.getServiceConfiguration(srh, DEFAULT_LICENSE_SERVICE);
					
        },
		
		getDefaultLicense: function(elements) {
			var self = this;
			dojo.forEach(elements.properties, function(prop){
				if(prop.name == "default.licenseid") {
					self.defaultLicense = prop;
				}
			});
			
			if (this.defaultLicense) {
				var row = d.ce("tr");
                this.serverInfoTable.appendChild(row);
                var td = d.ce("td");
                row.appendChild(td);
                td.innerHTML = this.defaultLicense.displayName;
                td.className = "propColumn";
                var td = d.ce("td");
                row.appendChild(td);
                td.innerHTML = this.defaultLicense.overrideValue || this.messages.noDefault
				td.id = "defaultLicenseField"

				var dElement = document.createElement("a");
				dElement.className = "connectLink";
				dElement.onclick = dojo.hitch(this, "launchDefaultWizard");
				dElement.innerHTML = this.messages.setDefault;
				dojo.byId('defaultLicenseField').appendChild(dElement);
			}
				
		}
    });
	    
    //
    // Utility functions
    //
    
    var d = document;
    d.ce = document.createElement;
    
    function clearMessageTimeout(ctxObj, timeoutProp){
        var timeoutObj = ctxObj[timeoutProp];
        clearTimeout(timeoutObj);
        ctxObj[timeoutProp] = null;
    }
    
    function col(row, node){
        if (node === null || node === undefined) {
            node = "-";
        }
		if(dojo.isString(node) && node.indexOf("null") != -1) {
			node = node.replace("null","");
		}
        var td = d.ce("td");
        var isNumber = typeof node == 'number' || node instanceof Number;
        var isNumberCol = isNumber || node === messages.notApplicable || node === messages.unlimited;
        if (isNumberCol) {
            td.className = "num";
        }
        if (isNumber) {
            node = dojo.number.format("" + node);
        }
        var isArray = !dojo.isString(node) && dojo.isArrayLike(node);
        if (node.tagName === "A" && node.className === "num") {
            td.className = "num";
        }
        else 
            if (isArray) {
                td.className = "num inputCol";
            }
        row.appendChild(td);
        var nodeArr = isArray ? node : [node];
        dojo.forEach(nodeArr, function(node){
            if (dojo.isString(node)) {
                node = document.createTextNode(node);
            }
            td.appendChild(node);
        });
    }
    
    function getCountOrUnlimited(count){
        return count === -1 ? messages.unlimited : count;
    }
    
    function getCountString(count){
        return count === -1 ? messages.unlimited : (dojo.number.format(count));
    }
    
    function getUsedLink(calType){
        if (calType.used === 0) {
            return 0;
        }
        else {
            var a = document.createElement("a");
            a.href = "#action=com.ibm.team.repository.manageUserCALs&licenseType=" + calType.id;
            a.innerHTML = calType.used;
            a.className = "num";
            return a;
        }
    }
    
    function getIssuedLink(calType){
        var a = document.createElement("a");
        a.href = "#action=com.ibm.team.repository.admin.floatingLicenseUsage&licenseId=" + calType.id;
        a.innerHTML = messages.viewIssued;
        return a;
    }

    function findOperationCheckResults(licenseInfo, operationId){
        var result = null;
        if (licenseInfo.operationCheckResults) {
            dojo.forEach(licenseInfo.operationCheckResults, function(operationResult){
                if (operationResult.operationId == operationId) {
                    result = operationResult.allowed;
                }
            })
        }
        return result;
    }
    
})();
