/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.users.SimpleTab");

/*
 * Imports 
 */
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");


(function() {

/*
 * Aliases 
 */

/*
 * Class 
 */
dojo.declare("com.ibm.team.repository.web.ui.internal.admin.users.SimpleTab",
	[dijit._Widget, dijit._Templated], {

	// Properties //----------------------------------------------------------->

	tabID: null,
	tabTitle: "",
	callbackGetNode: null,
	callbackOnClick: null,
	contentNode: null,
	
	/*
	 * Templates
	 */	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/templates/SimpleTab.html"),
	
	/*
	 * Static Variable Container
	 */
	_static: function() {}, 	

	// Public Methods //------------------------------------------------------->

	/*
 	 * @param Object args
 	 * 
 	 * @return 
 	 */
	constructor: function(args) {
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */	
	postCreate: function() {
		this._initializeStaticVariables();
		this._initializeInstanceVariables();
		this._initializeWidget();
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	destroy: function() {
		
		// Title
		delete this.tabTitle;
		// Tab ID
		delete this.tabID;

		// OnClick Handler
		if (this._onClickHandler) {
			this.disconnect(this._onClickHandler);
			delete this._onClickHandler;
		}

		// Content Node
		// Do not destroy (do not own)
		// On Click Callback
		// Do not destroy (do not own)
		// Get Node Callback
		// Do not destroy (do not own)

		this.inherited(arguments);
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	select: function() {
		
		if (!this.contentNode &&
			this.callbackGetNode) {
			this.contentNode = this.callbackGetNode(this.tabID);
		}
		if (!this.contentNode) {
			// Error
			this._logError(new Error("Undefined tab content node"));
			return;
		}
		
		dojo.addClass(this._tab, "selectedTab");
		this._updateVisibility(this.contentNode, true);
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	deselect: function() {
		
		if (!this.contentNode) {
			// Error
			this._logError(new Error("Undefined tab content node"));
			return;
		}		
		
		dojo.removeClass(this._tab, "selectedTab");
		this._updateVisibility(this.contentNode, false);
	},

	// Private Methods //------------------------------------------------------>

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initializeStaticVariables: function() {

		if (this._static._instanceCount) {
			this._static._instanceCount++;
			return;
		}

		this._static._instanceCount = 1;

	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initializeInstanceVariables: function() {
		this._onClickHandler = null;
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initializeWidget: function() {
		// Set tab title
		var titleText = document.createTextNode(this.tabTitle);
		this._link.appendChild(titleText);

		// Add click listener
		if (this.callbackOnClick) {
			this._onClickHandler = this.connect(this._link, "onclick", this._handleEventOnClick);
		}
	},
	
	/*
 	 * @param Event event
 	 * 
 	 * @return void
 	 */	
	_handleEventOnClick: function(event) {
		dojo.stopEvent(event);
		this.callbackOnClick(this);
	},	
	
	/*
 	 * @param Element node
 	 * @param boolean show
 	 * 
 	 * @return void
 	 */	
	_updateVisibility: function(node, show) {
		if (show) {
			node.style.display = "block";
		} else {
			node.style.display = "none";
		}
	},		
	
	/*
 	 * @param Error
 	 * 
 	 * @return void
 	 */
	_logError: function(errorObject) {
		
		var className = "";
		if (this.declaredClass) {
			className = this.declaredClass + ": ";
		} 
		
		var lineNumber = "";
		if (errorObject.lineNumber) {
			lineNumber = "(line " + errorObject.lineNumber + "): ";
		} 
		
		var errorMessage = "[ERROR]: " +
							className +
							lineNumber +
							errorObject.message;
							
		console.error(errorMessage);
	},		
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_noop: function() {
	}		
	
});

})();
